@@ -1,7 +1,7 @@
 /*
  * lftp and utils
  *
- * Copyright (c) 1996-2000 by Alexander V. Lukyanov (lav@yars.free.net)
+ * Copyright (c) 1996-2004 by Alexander V. Lukyanov (lav@yars.free.net)
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -18,7 +18,7 @@
  * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  */
 
-/* $Id: FileAccess.cc,v 1.103 2002/08/09 11:43:41 lav Exp $ */
+/* $Id: FileAccess.cc,v 1.115 2004/04/16 14:27:34 lav Exp $ */
 
 #include <config.h>
 
@@ -27,16 +27,12 @@
 #include <sys/types.h>
 #include <sys/socket.h>
 #include <netinet/in.h>
-#ifdef HAVE_NETINET_TCP_H
-# include <netinet/tcp.h>
-#endif
 #include <errno.h>
 #include <ctype.h>
 #include <assert.h>
 #include "ascii_ctype.h"
 #include <fcntl.h>
 #include "LsCache.h"
-#include "xalloca.h"
 #include "log.h"
 #include "url.h"
 #include "misc.h"
@@ -81,6 +77,7 @@ void FileAccess::Init()
    norest_manual=false;
    location=0;
    suggested_filename=0;
+   array_ptr=array_cnt=0;
 
    url=0;
 
@@ -148,40 +145,14 @@ void  FileAccess::DebugPrint(const char *prefix,const char *str,int level)
 {
    if(!Log::global)
       return;
-
-   char *msg=(char*)alloca(xstrlen(hostname)+strlen(prefix)+strlen(str)+25);
-
-   do
-   {
-      if(Log::global->IsTTY())
-	 strcpy(msg,prefix);
-      else
-      {
-	 sprintf(msg,"[%d] ",(int)getpid());
-	 if(hostname)
-	 {
-	    strcat(msg,hostname);
-	    strcat(msg," ");
-	 }
-	 strcat(msg,prefix);
-      }
-
-      int msglen=strlen(msg);
-      while(*str && *str!='\n')
-      {
-	 if(!(*str=='\r' && (str[1]=='\n' || str[1]=='\0')))
-	    msg[msglen++]=*str;
-	 str++;
-      }
-      msg[msglen++]='\n';
-      msg[msglen]=0;
-
-      Log::global->Write(level,msg);
-
-      if(!*str++)
-         break;
-   }
-   while(*str);
+   int len=strlen(str);
+   if(len>0 && str[len-1]=='\n')
+      len--;
+   if(len>0 && str[len-1]=='\r')
+      len--;
+   char *buf=string_alloca(strlen(prefix)+len+2);
+   sprintf(buf,"%s%.*s\n",prefix,len,str);
+   Log::global->Write(level,buf);
 }
 
 int   FileAccess::Poll(int fd,int ev)
@@ -362,6 +333,7 @@ void FileAccess::Close()
    opt_date=0;
    opt_size=0;
    array_for_info=0;
+   array_ptr=array_cnt=0;
    entity_size=NO_SIZE;
    entity_date=NO_DATE;
    ascii=false;
@@ -465,12 +437,9 @@ void FileAccess::Login(const char *user1,const char *pass1)
       }
       if(pass==0 && hostname) // still no pass? Try .netrc
       {
-	 NetRC::Entry *nrc=NetRC::LookupHost(hostname);
+	 NetRC::Entry *nrc=NetRC::LookupHost(hostname,user);
 	 if(nrc)
-	 {
-	    if(nrc->user && !strcmp(nrc->user,user))
-	       pass=xstrdup(nrc->pass);
-	 }
+	    pass=xstrdup(nrc->pass);
       }
    }
    ResetLocationData();
@@ -694,7 +663,7 @@ void FileAccess::Chdir(const char *path,bool verify)
    {
       if(cwd && cwd[0])
       {
-	 if(cwd[strlen(cwd)-1]=='/')
+	 if(last_char(cwd)=='/')
 	    sprintf(newcwd,"%s%s",cwd,path);
 	 else
 	    sprintf(newcwd,"%s/%s",cwd,path);
@@ -752,11 +721,7 @@ void FileAccess::SetSuggestedFileName(const char *fn)
       return;
 
    // don't allow subdirectories.
-   char *slash=strchr(fn,'/');
-   if(slash)
-      return;
-   slash=strchr(fn,'\\');
-   if(slash)
+   if(strchr(fn,'/') || strchr(fn,'\\'))
       return;
    for(int i=0; fn[i]; i++)
    {
@@ -863,6 +828,7 @@ bool FileAccess::NotSerious(int e)
 {
    switch(e)
    {
+   case(EPIPE):
    case(ETIMEDOUT):
 #ifdef ECONNRESET
    case(ECONNRESET):
@@ -933,7 +899,7 @@ DirList *FileAccess::MakeDirList(ArgV *a) { if(a) delete a; return 0; }
 
 DirList::~DirList()
 {
-   Delete(buf);
+   delete buf;
    if(args)
       delete args;
 }
@@ -941,7 +907,11 @@ DirList::~DirList()
 void FileAccess::CleanupAll()
 {
    for(FileAccess *o=chain; o!=0; o=o->next)
+   {
+      Enter(o);
       o->CleanupThis();
+      Leave(o);
+   }
 }
 
 FileAccess *FileAccess::NextSameSite(FA *scan)
@@ -1126,10 +1096,17 @@ void ListInfo::SetExclude(const char *p,PatternSet *x)
 #else
 # define _fish
 #endif
+#ifndef MODULE_PROTO_SFTP
+# include "SFtp.h"
+# define _sftp SFtp::ClassInit()
+#else
+# define _sftp
+#endif
 void FileAccess::ClassInit()
 {
    _ftp;
    _file;
    _http;
    _fish;
+   _sftp;
 }
