@@ -18,7 +18,7 @@
  * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  */
 
-/* $Id: Http.h,v 1.40 2004/01/21 08:38:09 lav Exp $ */
+/* $Id: Http.h,v 1.42 2004/08/12 09:58:11 lav Exp $ */
 
 #ifndef HTTP_H
 #define HTTP_H
@@ -59,6 +59,8 @@ class Http : public NetAccess
    bool post;
    char *post_data;
    void SendAuth();
+   void SendCacheControl();
+   void SendBasicAuth(const char *tag,const char *auth);
    void SendBasicAuth(const char *tag,const char *u,const char *p);
    void SendRequest(const char *connection,const char *f);
    void SendRequest(const char *connection=0)
