@@ -18,7 +18,7 @@
  * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  */
 
-/* $Id: StatusLine.cc,v 1.31 2004/05/04 11:58:34 lav Exp $ */
+/* $Id: StatusLine.cc,v 1.32 2004/06/12 07:55:43 lav Exp $ */
 
 #include <config.h>
 #include <sys/types.h>
@@ -114,21 +114,22 @@ void StatusLine::Show(const char *f,...)
 
 void StatusLine::ShowN(const char *const* newstr,int n)
 {
-   if(update_delayed && update_timer.Stopped())
-   {
-      update(newstr,n);
-      update_delayed=false;
-      return;
-   }
-
    if(!update_delayed && shown.IsEqual(newstr,n))
       return;
    if(update_delayed && to_be_shown.IsEqual(newstr,n))
       return;
 
-   /* not yet */
-   to_be_shown.Assign(newstr,n);
-   update_delayed=true;
+   if(!update_timer.Stopped())
+   {
+      /* not yet */
+      to_be_shown.Assign(newstr,n);
+      update_delayed=true;
+   }
+   else
+   {
+      update(newstr,n);
+      update_delayed=false;
+   }
 }
 
 const char *StatusLine::to_status_line = get_string_term_cap("tsl", "ts");
