@@ -1,7 +1,7 @@
 /*
  * lftp and utils
  *
- * Copyright (c) 1996-2001 by Alexander V. Lukyanov (lav@yars.free.net)
+ * Copyright (c) 1996-2004 by Alexander V. Lukyanov (lav@yars.free.net)
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -18,7 +18,7 @@
  * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  */
 
-/* $Id: lftp.cc,v 1.45.2.2 2002/01/15 16:20:26 lav Exp $ */
+/* $Id: lftp.cc,v 1.62 2004/09/08 08:41:50 lav Exp $ */
 
 #include <config.h>
 
@@ -32,11 +32,8 @@
 #include <sys/stat.h> // for mkdir()
 #include "xstring.h"
 #include <fcntl.h>
-#ifdef HAVE_LOCALE_H
-# include <locale.h>
-#endif
+#include <locale.h>
 
-#include "xalloca.h"
 #include "xmalloc.h"
 #include "alias.h"
 #include "CmdExec.h"
@@ -47,6 +44,11 @@
 #include "DummyProto.h"
 #include "ResMgr.h"
 #include "LsCache.h"
+#include "DirColors.h"
+#include "IdNameCache.h"
+#include "LocalDir.h"
+#include "ConnectionSlot.h"
+#include "misc.h"
 
 #include "confpaths.h"
 
@@ -75,6 +77,11 @@ void WaitDone(CmdExec *exec)
    }
 }
 
+ResDecl res_save_cwd_history
+   ("cmd:save-cwd-history","yes",ResMgr::BoolValidate,ResMgr::NoClosure);
+ResDecl res_save_rl_history
+   ("cmd:save-rl-history","yes",ResMgr::BoolValidate,ResMgr::NoClosure);
+
 class ReadlineFeeder : public CmdFeeder
 {
    bool tty:1;
@@ -82,17 +89,46 @@ class ReadlineFeeder : public CmdFeeder
    bool add_newline:1;
    char *to_free;
    int eof_count;
+   char *for_history;
+
+   static bool readline_inited;
+   void readline_init()
+   {
+      if(readline_inited)
+	 return;
+      readline_inited=true;
+      lftp_readline_init();
+      lftp_rl_read_history();
+      if(for_history)
+      {
+	 lftp_add_history_nodups(for_history);
+	 xfree(for_history);
+	 for_history=0;
+      }
+   }
+
 public:
-   ReadlineFeeder()
+   ReadlineFeeder(const ArgV *args)
    {
       tty=isatty(0);
       ctty=(tcgetpgrp(0)!=(pid_t)-1);
       to_free=0;
       eof_count=0;
+      for_history=0;
+      if(args && args->count()>1)
+	 for_history=args->CombineQuoted();
    }
    virtual ~ReadlineFeeder()
    {
+      if(readline_inited)
+      {
+	 if(res_save_cwd_history.QueryBool(0))
+	    cwd_history.Save();
+	 if(res_save_rl_history.QueryBool(0))
+	    lftp_rl_write_history();
+      }
       xfree(to_free);
+      xfree(for_history);
    }
    bool RealEOF()
    {
@@ -116,6 +152,8 @@ public:
       char *cmd_buf;
       if(tty)
       {
+	 readline_init();
+
 	 if(ctty) // controlling terminal
 	 {
 	    pid_t term_pg=tcgetpgrp(0);
@@ -164,7 +202,7 @@ public:
 	 else
 	    eof_count=0;
       }
-      else
+      else // not a tty
       {
 	 if(exec->interactive)
 	 {
@@ -182,8 +220,7 @@ public:
 
       ::completion_shell=0;
 
-      if(cmd_buf &&
-	    (*cmd_buf==0 || cmd_buf[strlen(cmd_buf)-1]!='\n'))
+      if(cmd_buf && last_char(cmd_buf)!='\n')
       {
 	 exec->Timeout(0);
 	 add_newline=true;
@@ -197,6 +234,7 @@ public:
 	 lftp_rl_clear();
       }
 };
+bool ReadlineFeeder::readline_inited;
 
 static void sig_term(int sig)
 {
@@ -246,6 +284,9 @@ static void move_to_background()
 	    if(fd!=1 && fd!=2)
 	       close(fd);
 	 }
+	 Log::global->ShowPID();
+	 Log::global->ShowTime();
+	 Log::global->ShowContext();
       }
       close(0);	  // close stdin.
       open("/dev/null",O_RDONLY); // reopen it, just in case.
@@ -279,6 +320,18 @@ static void move_to_background()
    }
 }
 
+int lftp_slot(int count,int key)
+{
+   if(!top_exec)
+      return 0;
+   char slot[2];
+   slot[0]=key;
+   slot[1]=0;
+   top_exec->ChangeSlot(slot);
+   lftp_rl_set_prompt(top_exec->MakePrompt());
+   return 0;
+}
+
 void  source_if_exist(CmdExec *exec,const char *rc)
 {
    if(access(rc,R_OK)!=-1)
@@ -295,9 +348,6 @@ static void tty_clear()
       top_exec->pre_stdout();
 }
 
-ResDecl res_save_cwd_history
-   ("cmd:save-cwd-history","yes",ResMgr::BoolValidate,ResMgr::NoClosure);
-
 int   main(int argc,char **argv)
 {
 #ifdef SOCKS4
@@ -314,11 +364,11 @@ int   main(int argc,char **argv)
    ResMgr::ClassInit();
    FileAccess::ClassInit();
 
-   lftp_readline_init();
-
    hook_signals();
 
-   top_exec=new CmdExec(new DummyProto());
+   LocalDirectory *cwd=new LocalDirectory;
+   cwd->SetFromCWD();
+   top_exec=new CmdExec(new DummyProto(),cwd);
    top_exec->SetStatusLine(new StatusLine(1));
    Log::global->SetCB(tty_clear);
 
@@ -346,13 +396,9 @@ int   main(int argc,char **argv)
 
    ArgV *args=new ArgV(argc,argv);
    args->setarg(0,"lftp");
-   if(args->count()>1)
-   {
-      char *line=args->CombineQuoted();
-      lftp_add_history_nodups(line);
-      xfree(line);
-   }
-   lftp_feeder=new ReadlineFeeder;
+
+   lftp_feeder=new ReadlineFeeder(args);
+
    top_exec->ExecParsed(args);
 
    WaitDone(top_exec);
@@ -360,9 +406,6 @@ int   main(int argc,char **argv)
    top_exec->AtExit();
    WaitDone(top_exec);
 
-   if(res_save_cwd_history.QueryBool(0))
-      cwd_history.Save();
-
    if(Job::NumberOfJobs()>0)
    {
       top_exec->SetInteractive(false);
@@ -372,12 +415,19 @@ int   main(int argc,char **argv)
    int exit_code=top_exec->ExitCode();
    SMTask::Delete(top_exec);
    top_exec=0;
+   ConnectionSlot::Cleanup();
    SessionPool::ClearAll();
    LsCache::Flush();
    ProcWait::DeleteAll();
+   DirColors::DeleteInstance();
+   IdNameCacheCleanup();
+   SignalHook::Cleanup();
+   Log::Cleanup();
+   SMTask::Cleanup();
 
+   // the tasks left: LsCache::ExpireHelper
    int task_count=SMTask::TaskCount();
-   if(task_count>3)
+   if(task_count>1)
       printf("WARNING: task_count=%d\n",task_count);
 
    return exit_code;
