@@ -18,7 +18,7 @@
  * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  */
 
-/* $Id: misc.h,v 1.34 2004/04/16 14:27:36 lav Exp $ */
+/* $Id: misc.h,v 1.35 2004/11/15 09:27:57 lav Exp $ */
 
 #ifndef MISC_H
 #define MISC_H
@@ -115,4 +115,7 @@ char *dirname_modify(char *fn);
 /* returns last character of string or \0 if string is empty */
 char last_char(const char *str);
 
+int  base64_length (int len);
+void base64_encode (const char *s, char *store, int length);
+
 #endif // MISC_H
